﻿using Infragistics.Controls.Maps;
using Infragistics.DragDrop;

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml.Linq;
using WPF_xamOrgChart;

namespace WPF_xamOrgChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        public MainWindow()
        {
            InitializeComponent();          
        }

       
        private void button1_Click(object sender, RoutedEventArgs e)
        {
            ObservableCollection<Department> depts = OrgChart.ItemsSource as ObservableCollection<Department>;
            depts[0].EmployeePositions[0].Employees.Add(new Employee("firstname", "lastname", "title", "/Images/People/chinchilla.png", new ObservableCollection<Employee>()));
        }

        private void OrgChart_NodeControlAttachedEvent(object sender, OrgChartNodeEventArgs e)
        {

            DragSource dragSource = new DragSource();
            dragSource.IsDraggable = true;
            dragSource.DragStart += Node_DragStart;
            dragSource.Drop += Node_Drop;

            //Make the node a drag source.
            DragDropManager.SetDragSource(e.Node, dragSource);

            //Make the node a drop target.
            DragDropManager.SetDropTarget(e.Node, new DropTarget()
            {
                IsDropTarget = true,
            });          

        }

        private void OrgChart_NodeControlDetachedEvent(object sender, OrgChartNodeEventArgs e)
        {

        }   

        private void Node_DragStart(object sender, DragDropStartEventArgs e)
        {
            //Assing the Employee of the dragged node as data context for the Dragged Item.
            //This is needed for the DragTemplate.
            e.Data = ((OrgChartNodeControl)e.DragSource).Node.Data;

        }

        // Perform a drop and update the underlying data source.
        private void Node_Drop(object sender, DropEventArgs e)
        {
            //Get the dragged OrgChartNode object.
            OrgChartNode dragSourceNode = ((OrgChartNodeControl)e.DragSource).Node as OrgChartNode;

            //Get the dragged employee.
            Employee dragSource = dragSourceNode.Data as Employee;

            //Get the collection with sibling Employees.
            IList<Employee> siblingsCollection;
            
            if (dragSourceNode.ParentNode.Data is EmployeePosition)
                siblingsCollection = (dragSourceNode.ParentNode.Data as EmployeePosition).Employees;

            else
            {
                Employee superiorEmployee = (dragSourceNode.ParentNode.Data as Employee);
                siblingsCollection = superiorEmployee.Reports;
            }
           

            //Perform the drop.
            if (e.DropTarget is OrgChartNodeControl)
            {
                Employee dropTarget = ((OrgChartNodeControl)e.DropTarget).Node.Data as Employee;

                //Prevent drop on self.
                if (dragSource == dropTarget) return;

                //Check if the drop target already contains the dragged item.
                if (dropTarget.Reports != null)
                {
                    if (dropTarget.Reports.Contains(dragSource)) return;
                }
                else
                    dropTarget.Reports = new ObservableCollection<Employee>();

                //Check for a recursive relation between the drag source and the drop target.
                int levels = OrgChart.ActualDepth - dragSourceNode.Level;
                if (dragSourceNode.HighlightChildren(levels)
                    .Select(node => node.Data)
                    .Contains(dropTarget)) return;

                //Remove the dragged node from its parent.
                siblingsCollection.Remove(dragSource);

                //Add the dragged node to the drop target node.
                dropTarget.Reports.Add(dragSource as Employee);
            }
            else //if the Drop Target is the XamOrgChart
            {
                ObservableCollection<Department> depts = (((XamOrgChart)e.DropTarget).ItemsSource as ObservableCollection<Department>);

                //Check if the drop target already contains the dragged item.
                if (depts[0].EmployeePositions[0].Employees.Contains(dragSource)) return;

                //if (EmployeesVM.Employees.Contains(dragSource)) return;
                ObservableCollection<Employee> employees = depts[0].EmployeePositions[0].Employees;

                //Remove the dragged node from its parent.
                siblingsCollection.Remove(dragSource);

                //Add the dragged node to the drop target node.
                employees.Add(dragSource);
            }
        }

           
    }
}
   

